# -*- coding: utf-8 -*-
import os
from qpid.messaging import *
import MySQLdb
import traceback
import datetime
import time
import json
import marshal
import logging, logging.config

logging.config.fileConfig('maint_log.ini')
progressDir = "/linuxcash/cash/exchangesystems/progress"
progressFileName = os.path.join(progressDir,"counter_maint_prices.json")
log = logging.getLogger('su.artix.pricedeleteservice')

def chunks(chunkable, n):
    for i in xrange(0, len(chunkable), n):
        yield chunkable[i:i+n]

def createProgressFile():
    if not os.path.exists(progressDir):
        return
    try:
        open(progressFileName, 'a').close()
        log.info('Создали файл прогресса %s.' % progressFileName)
    except Exception, e:
        log.error('Ошибка создания %s. %s' % (progressFileName ,e))

def deleteProgressFile():
    if not os.path.exists(progressFileName):
        return
    try:
        os.remove(progressFileName)
        log.info('Удалили файл прогресса %s.' % progressFileName)
    except Exception, e:
        log.error('Ошибка удаления %s. %s' % (progressFileName ,e))

def sendMessage(resultList):
    if resultList:
        createProgressFile()
        if start():
            add_package(resultList)
            stop()
        deleteProgressFile()
    else:
        log.info('Список команд для отправления - пустой')

def calculateData():
    try:
        data_file = open('/linuxcash/cash/conf/maint.json')
        data = json.load(data_file)
        resultCount = data["prices"]["pricesCount"] if data.has_key("prices") and data["prices"].has_key("pricesCount")  else 150
        timeout = data["prices"]["timeout"] if data.has_key("prices") and data["prices"].has_key("timeout") else 240
        log.info('Начало выборки данных из бд')
        conn = MySQLdb.connect(host='localhost', user='root', passwd='', db="dictionaries")
        cur = conn.cursor()
        cur.execute('set names utf8')   
        cur.execute("SELECT code FROM tmc")
        codeList = [item[0] for item in cur.fetchall()]
        log.info('Выбрали все коды из таблицы tmc')
        cur.execute("SELECT barcode FROM barcodes")
        tempList = [item[0] for item in cur.fetchall()]
        log.info('Выбрали все штрих-коды из таблицы barcodes')
        codeList.extend(tempList)
        log.info('Собрали два списка вместе')
        listChunk = 50
        commandList = []
        cur.close()
        conn.close()
        for row in chunks(codeList, listChunk):
            conn = MySQLdb.connect(host='localhost', user='root', passwd='', db="dictionaries")
            cur = conn.cursor()
            cur.execute('set names utf8')
            cur.execute("SELECT DISTINCT(barcode) FROM dictionaries.prices WHERE effectivedateend < now()")
            result = [{'command': 'deleteOverduePrices', 'barcode': str(item[0])} for item in cur.fetchall()]
            row = filter(None, row)
            query_values = ', '.join("'%s'" % it for it in row)
            query = "SELECT barcode, max(effectivedate), effectivedateend FROM dictionaries.prices \
                    WHERE barcode IN (%s) AND effectivedate < now() AND (effectivedateend > now() OR effectivedateend IS NULL) \
                    GROUP BY barcode, effectivedateend HAVING count(barcode) > 1" % str(query_values)

            cur.execute(query)
            result += [
                {'command': 'deletePriceByDate', 
                 'barcode': str(item[0]), 
                 'effectivedate': str(item[1]), 
                 'effectivedateend': str(item[2]) if item[2] else None} 
                       for item in cur.fetchall()] 
            cur.close()
            conn.close()
            if result:
                for i in result:
                    log.info(str(i))
                commandList.extend(result)
                log.info('Сформировали %d команд для nes. Общий размер пакета %d' % (len(result), len(commandList)))
                result = []

                if len(commandList) > resultCount:
                    for row in chunks(commandList, resultCount):
                        log.info('Посылаем %d команд в nes' % len(row))
                        sendMessage(resultList=row)
                        time.sleep(timeout)
                    commandList = []
        if commandList:
            sendMessage(commandList)
    except Exception, e:
        log.error('Ошибка : %s' % e)
    finally:
        if conn.open:
            cur.close()
            conn.close()

def checkQueueParams(target, queueName, max_size, file_size, file_count, max_count):
    if os.name == 'posix':
        sess = None
        try:
            sess = Session()
            broker = sess.addBroker(target=target)
            queues = sess.getObjects(_class="queue", _package="org.apache.qpid.broker")
            for q in queues:
                if q.name == queueName:
                    args = q.arguments
                    if args.get('qpid.max_size', None) != max_size or args.get('qpid.file_size', None) != file_size or \
                        args.get('qpid.file_count', None) != file_count or args.get('qpid.max_count', None) != max_count:
                        if q.msgDepth == 0:
                            # если настройки очереди изменились, то удаляем эту очередь, чтобы создать с новыми настройками
                            broker.getAmqpSession().queue_delete(queue=q.name, if_empty=True, if_unused=True)
                    break
        except Exception, e:
            log.error('Ошибка удалении очереди %s . %s' % (queueName, e))
        finally:
            if sess:
                sess.close()

def start():
    global connection
    global session
    global sender
    global startMessageId
    global objectsCount
    objectsCount = 0
    try:
        registryPath = '/linuxcash/cash/data/cash.reg'
        with open(registryPath) as data_file:
            data = json.load(data_file)
            cashCode = str(data['cashCode'])

        broker = 'localhost:5672'
        connection = Connection(broker, reconnect=True)
        if not connection.opened():
            connection.open()
        session = connection.session(transactional=True)
        queueName = 'artix.loaddict.cash'
        fileSize = 128
        fileCount = 8
        maxSize = 104857600
        maxCount = 3000
        #checkQueueParams(broker, queueName, maxSize, fileSize, fileCount, maxCount)
        sender = session.sender("%s; {create: always, node: { type: queue, durable: True, x-declare: {arguments: {'qpid.file_size':%s, 'qpid.file_count':%s, 'qpid.max_size':%s, 'qpid.max_count':%s}}} }" % (queueName, fileSize, fileCount, maxSize, maxCount), durable=True)
        prop = {}
        prop['type'] = 'control'
        prop['control'] = 'begin_tran'
        prop['dictCashLoadStateId'] = None
        prop['cashCode'] = cashCode
        prop['statusServer'] = 'localhost:8080'
        startMessageId = hex(hash(datetime.datetime.now()))
        prop['startMessageId'] = startMessageId
        log.info('Отправляем сообщение - начало транзакции')
        sender.send(Message(content=None, id=startMessageId, properties=prop, durable=True))
        return True
    except Exception, e:
        log.error('Ошибка: %s' % e)
        log.error(traceback.format_exc())
    return False

def add_package(listObjects):
    global objectsCount
    objectsCount += len(listObjects)
    log.debug('Отправляем пакет объектов справочника в очередь')
    prop = {'type': 'datagram', 'serialType': 'marshal'}
    body = marshal.dumps(listObjects)
    sender.send(Message(content=body, id=hex(hash(datetime.datetime.now())), properties=prop, durable=True))

def stop():
    prop = {'type': 'control', 'control':'end_tran'}
    log.info('Отправляем сообщение - конец транзакции')
    sender.send(Message(content=None, id=hex(hash(datetime.datetime.now())), properties=prop, durable=True))
    try:
        session.commit()
    except Exception, e:
        log.info('Ошибка: %s' % e)
        session.rollback()
        try:
            log.info('Закрываем connection')
            connection.close()
        except Exception, er:
            log.info('Ошибка: %s' % er)
        return False
    connection.close()
    log.info('Загрузка прошла успешно. Разблокируем загрузку в очередь для магазина')
    saveProgress()
    return True

def saveProgress():
    if not os.path.exists(progressDir):
        return
    data = {"objectsCount": objectsCount}
    filePath = os.path.join(progressDir, "counter_%s.json" % startMessageId)
    with open(filePath, 'w') as outfile:
        json.dump(data, outfile)

def main():
    log.info('Сервис maint_prices начал работу')
    calculateData()
    log.info('Сервис maint_prices закончил работу')

if __name__ == '__main__':
    main()